@@ -22,11 +22,11 @@ import threading
 import time
 from urllib import parse as urlparse
 
-import eventlet
+import requests
+
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log
-import requests
 
 from ironic_python_agent.api import app
 from ironic_python_agent import config
@@ -108,7 +108,7 @@ class IronicPythonAgentHeartbeater(threading.Thread):
         self.agent.set_agent_advertise_addr()
 
         while self._run_next():
-            eventlet.sleep(0)
+            time.sleep(0.1)
 
     def _run_next(self):
         # The logic here makes sure we don't wait exactly 5 seconds more or
@@ -258,7 +258,6 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
         # Allows this to be turned on by the conductor while running,
         # in the event of long running ramdisks where the conductor
         # got upgraded somewhere along the way.
-        self.agent_token_required = cfg.CONF.agent_token_required
         self.generated_cert = None
 
     def get_status(self):
@@ -484,7 +483,7 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
             self.heartbeater.start()
         try:
             while self.serve_api:
-                eventlet.sleep(0.1)
+                time.sleep(0.1)
         except KeyboardInterrupt:
             LOG.info('Caught keyboard interrupt, exiting')
         self.api.stop()
@@ -500,6 +499,9 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
 
         # Update config with values from Ironic
         config = content.get('config', {})
+        if config.get('agent_containers'):
+            for opt, val in config['agent_containers'].items():
+                cfg.CONF.set_override(opt, val, group='container')
         if config.get('metrics'):
             for opt, val in config.items():
                 setattr(cfg.CONF.metrics, opt, val)
@@ -515,8 +517,6 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
         md5_allowed = config.get('agent_md5_checksum_enable')
         if md5_allowed is not None:
             cfg.CONF.set_override('md5_enabled', md5_allowed)
-        if config.get('agent_token_required'):
-            self.agent_token_required = True
         token = config.get('agent_token')
         if token:
             if len(token) >= 32:
@@ -532,11 +532,15 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
                             'intended and the deployment may fail '
                             'depending on settings in the ironic '
                             'deployment.')
-                if not self.agent_token and self.agent_token_required:
-                    LOG.error('Ironic is signaling that agent tokens '
-                              'are required, however we do not have '
-                              'a token on file. '
-                              'This is likely **FATAL**.')
+                if not self.agent_token:
+                    LOG.error('We do not have a token on file '
+                              'from the Ironic deployment, and '
+                              'one should be on file. '
+                              'Possible external agent restart '
+                              'outside of Ironic\'s process. '
+                              'This is **FATAL**.')
+                    self.serve_api = False
+                    self.lockdown = True
             else:
                 LOG.info('An invalid token was received.')
         if self.agent_token and not self.standalone:
@@ -603,7 +607,7 @@ class IronicPythonAgent(base.ExecuteCommandMixin):
                           'found, please check your pxe append parameters.')
 
         in_rescued_mode = os.path.exists('/etc/.rescued')
-        if not in_rescued_mode:
+        if not in_rescued_mode and self.serve_api:
             self.serve_ipa_api()
         else:
             # NOTE(cid): In rescued state, we don't call _lockdown_system() as
